using System;
using System.Collections;
using System.ComponentModel;
using System.Text;

using Syncfusion.Collections;
using Syncfusion.Collections.BinaryTree;
using Syncfusion.Diagnostics;
using Syncfusion.Grouping;

using ISummary = Syncfusion.Collections.BinaryTree.ITreeTableSummary;

namespace TestGroupingGrid.CustomSummary
{
	/// <summary>
	/// Simple summary example: Counts total sum of entries. 
	/// </summary>
	public sealed class TotalSummary : SummaryBase
	{
		double _total;

		public static readonly TotalSummary Empty = new TotalSummary(0);
		
		/// <summary>
		/// Assign this CreateSummaryDelegate handler method to SummaryDescriptor.CreateSummaryMethod 
		/// </summary>
		/// <param name="sd"></param>
		/// <param name="record"></param>
		/// <returns></returns>
		public static ISummary CreateSummaryMethod(SummaryDescriptor sd, Record record)
		{
//			object obj = sd.GetValue(record);
//			bool isNull = (obj == null || obj is DBNull);
//			if (isNull)
//				return Empty;
//			else
//			{
//				double val = Convert.ToDouble(obj);
//				return new TotalSummary(val);
//			}
			return new TotalSummary(1);
		}

		/// <summary>
		/// Initializes a new summary object.
		/// </summary>
		/// <param name="total"></param>
		public TotalSummary(double total)
		{
			_total = total;
		}

		/// <summary>
		/// The running total of this summary
		/// </summary>
		public double Total
		{
			get
			{
				return _total;
			}
		}

		/// <summary>
		/// Combines this summary information with another objects summary and returns a new object.  
		/// </summary>
		/// <param name="other"></param>
		/// <returns></returns>
		/// <remarks>
		/// This method must honor the immutable characteristics of summary objects and return 
		/// a new summary object instead of modifying an existing summary object.
		/// </remarks>
		public override SummaryBase Combine(SummaryBase other)
		{
			return Combine((TotalSummary) other);
		}

		/// <summary>
		/// Combines this summary information with another objects summary and returns a new object.  
		/// </summary>
		/// <param name="other"></param>
		/// <returns></returns>
		/// <remarks>
		/// This method must honor the immutable characteristics of summary objects and return 
		/// a new summary object instead of modifying an existing summary object.
		/// </remarks>
		public TotalSummary Combine(TotalSummary other)
		{
			// Summary objects are immutable. That means properties cannot be modified for an 
			// existing object. Instead every time a change is made a new object must be created (just like 
			// System.String). 
			//
			// This allows following optimization: return existing summary object if either one of the values is 0. -->
			if (other.Total == 0)
				return this;
			else if (Total == 0)
				return other;
				// <-- end of optimization
			else
				return new TotalSummary(this.Total + other.Total);
		}

		/// <override/>
		public override string ToString()
		{
			return String.Format("Total = {0:0.00}", Total);
		}
	}

}
